﻿using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;

namespace GoBusExperimental
{
    public class Button : GoModule
    {
        Guid _moduleGuid = new Guid(new byte[] { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }); // button
        public override Guid ModuleGuid { get { return _moduleGuid; } }

        private bool _isPressed = false;

        public delegate void ButtonEventHandler(object sender, bool isPressed);
        public event ButtonEventHandler ButtonPressed;
        public event ButtonEventHandler ButtonReleased;

        public Button()
            : base()
        {
        }

        public Button(IGoPort goPort)
            : base(goPort)
        {
        }

        protected override void Initialize()
        {
            _goPort.GpioPinInterruptMode = Port.InterruptMode.InterruptEdgeBoth;
            _goPort.GpioPinMode = GoPortGpioPinMode.IRQ;
            _goPort.GpioPinGlitchFilter = true;
            _goPort.OnGpioPinInterrupt += _interruptPort_OnInterrupt;

            base.Initialize();

            // read initial button state
            _isPressed = !_goPort.GpioRead();
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                // clean up any managed code objects
                _goPort.OnGpioPinInterrupt -= _interruptPort_OnInterrupt;
            }
            // clean up any unmanaged code objects

            // dispose of our base object
            base.Dispose(disposing);
        } 

        #region LegacyCompatibility
        // NOTE: this is legacy support for projects using the 1.0.0 button class
        [System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)]
        public bool ButtonState
        {
            get
            {
                return IsPressed;
            }
        }
        #endregion

        public bool IsPressed
        {
            get
            {
                return _isPressed;
            }
        }

        void _interruptPort_OnInterrupt(uint pin, uint state, DateTime time)
        {
            _isPressed = (state == 0);
            if (!_isPressed)
            {
                // released
                if (ButtonReleased != null)
                    ButtonReleased(this, false);
            }
            else
            {
                // pressed
                if (ButtonPressed != null)
                    ButtonPressed(this, true);
            }
        }
    }
}
