using System;
using Microsoft.SPOT;
using System.Collections;

namespace GoBusExperimental
{
    // Most hub implementations can safely inherit from GoHubBase since it doesn't contain any hardware-specific code.

    public abstract class GoHubBase : IGoHub
    {
        protected readonly ArrayList _goPorts = new ArrayList();
        public virtual IGoPort[] GoPorts { get { return (IGoPort[])_goPorts.ToArray(typeof(IGoPort)); } }

        public IGoPort GetGoPort(int portNumber)
        {
            return GoPorts[portNumber - 1];
        }

        public IEnumerable GetGoPortsForModuleGuid(Guid guid, bool includeBoundPorts)
        {
            IGoPort goPort;
            Guid portGuid;

            for (int i = 0; i < GoPorts.Length; i++)
            {
                goPort = GoPorts[i];
                if (!includeBoundPorts && goPort.BoundModule != null)
                    continue;

                portGuid = goPort.GetConnectedModuleGuid();

                if (guid.Equals(portGuid))
                    yield return goPort;
            }
        }

        public virtual void OnGoPortEnumerating(IGoPort goPort)
        {
        }

        public virtual void OnGoPortEnumerated(IGoPort goPort, bool isBinding)
        {
        }

        public virtual void OnGoPortBinding(IGoPort goPort)
        {
        }

        public virtual void OnGoPortBound(IGoPort goPort)
        {
        }

        public virtual void OnGoPortUnbound(IGoPort goPort)
        {
        }
    }
}
