using System;
using Microsoft.SPOT;
using GoBusExperimental;
using System.Collections;
using SecretLabs.NETMF.Hardware;
using Microsoft.SPOT.Hardware;

namespace SampleApp
{
    class PWMHub : GoHub
    {
        private const uint UnboundPWM = 1;
        private const uint EnumeratingPWM = 10;
        private const uint BoundPWM = 100;

        private readonly ArrayList _goPortPWMLEDs = new ArrayList();

        public override void AddGoPort(IGoPort goPort, Cpu.Pin powerPin, Cpu.Pin ledPin)
        {
            base.AddGoPort(goPort, powerPin, Cpu.Pin.GPIO_NONE);
            var pwm = new SecretLabs.NETMF.Hardware.PWM(ledPin);
            pwm.SetDutyCycle(UnboundPWM);
            _goPortPWMLEDs.Add(pwm);
        }

        protected virtual void SetPortLEDDutyCycle(IGoPort goPort, uint dutyCycle)
        {
            var index = _goPorts.IndexOf(goPort);
            var pwm = (SecretLabs.NETMF.Hardware.PWM)_goPortPWMLEDs[index];
            pwm.SetDutyCycle(dutyCycle);
        }

        public override void OnGoPortEnumerating(IGoPort goPort)
        {
            base.OnGoPortEnumerating(goPort);
            SetPortLEDDutyCycle(goPort, EnumeratingPWM);
        }

        public override void OnGoPortEnumerated(IGoPort goPort, bool isBinding)
        {
            base.OnGoPortEnumerated(goPort, isBinding);
            SetPortLEDDutyCycle(goPort, UnboundPWM);
        }

        public override void OnGoPortBound(IGoPort goPort)
        {
            base.OnGoPortBound(goPort);
            SetPortLEDDutyCycle(goPort, BoundPWM);
        }

        public override void OnGoPortUnbound(IGoPort goPort)
        {
            base.OnGoPortUnbound(goPort);
            SetPortLEDDutyCycle(goPort, UnboundPWM);
        }
    }
}
